@@ -1,6 +1,6 @@
 package RT::Authen::ExternalAuth;
 
-our $VERSION = '0.20';
+our $VERSION = '0.23';
 
 =head1 NAME
 
@@ -8,21 +8,58 @@ RT::Authen::ExternalAuth - RT Authentication using External Sources
 
 =head1 DESCRIPTION
 
-A complete package for adding external authentication mechanisms
-to RT. It currently supports LDAP via Net::LDAP and External Database
-authentication for any database with an installed DBI driver.
+This module provides the ability to authenticate RT users against one or
+more external data sources at once. It will also allow information about
+that user to be loaded from the same, or any other available, source as
+well as allowing multple redundant servers for each method.
 
-It also allows for authenticating cookie information against an
-external database through the use of the RT-Authen-CookieAuth extension.
+The extension currently supports authentication and information from
+LDAP via the Net::LDAP module, and from any data source that an
+installed DBI driver is available for.
+
+It is also possible to use cookies set by an alternate application for
+Single Sign-On (SSO) with that application.  For example, you may
+integrate RT with your own website login system so that once users log
+in to your website, they will be automagically logged in to RT when they
+access it.
+
+=head1 INSTALLATION
+
+=over
+
+=item C<perl Makefile.PL>
+
+=item C<make>
+
+=item C<make install>
+
+May need root permissions
+
+=item Edit your F</opt/rt4/etc/RT_SiteConfig.pm>
+
+If you are using RT 4.2 or greater, add this line:
+
+    Plugin('RT::Authen::ExternalAuth');
+
+For RT 4.0, add this line:
+
+    Set(@Plugins, qw(RT::Authen::ExternalAuth) );
+
+or add C<RT::Authen::ExternalAuth> to your existing C<@Plugins> line.
+
+See L</CONFIGURATION> for additional configuration to add to your
+F<RT_SiteConfig.pm> file.
+
+=back
 
 =head1 UPGRADING
 
 If you are upgrading from an earlier version of this extension, you must
 remove the following files manually:
 
-    $RTHOME/local/plugins/RT-Authen-ExternalAuth/lib/RT/User_Vendor.pm
-    $RTHOME/local/lib/RT/User_Vendor.pm
-    $RTHOME/local/lib/RT/Authen/External_Auth.pm
+    /opt/rt4/local/plugins/RT-Authen-ExternalAuth/lib/RT/User_Vendor.pm
+    /opt/rt4/local/lib/RT/User_Vendor.pm
+    /opt/rt4/local/lib/RT/Authen/External_Auth.pm
 
 Otherwise you will most likely encounter an error about modifying a read
 only value and be unable to start RT.
@@ -31,103 +68,224 @@ You may not have all of these files.  It depends what versions you are
 upgrading between.
 
 If you are using a vendor packaged RT, your local directories are likely
-to be somewhere under /usr/local instead of in $RTHOME so you will need
+to be somewhere under /usr/local instead of in /opt/rt4 so you will need
 to visit Configuration -> Tools -> System Configuration to find your
 plugin root.
 
-=head2 VERSION NOTES
+=head1 CONFIGURATION
 
-If you are using RT 3.6, you want to use the 0.05 version.
+L<RT::Authen::ExternalAuth> provides a lot of flexibility with many
+configuration options.  The following desc these configuration options,
+and provides a complete example.
 
-If you are using RT 3.8.0 or 3.8.1, you may have trouble using this
-due to RT bugs related to plugins, but you may be able to use 0.08.
+=over 4
 
-0.08_02 or later will not work on 3.8.0 or 3.8.1
+=item C<$ExternalAuthPriority>
 
-If you are using RT 4.0.0 or greater, you must use at least 0.09
+The order in which the services defined in L</$ExternalSettings> should
+be used to authenticate users.  Once the user has been authenticated by
+one service, the rest are skipped.
 
-=head1 MORE ABOUT THIS MODULE 
+You should remove services you don't use. For example, if you're only
+using C<My_LDAP>, remove C<My_MySQL> and C<My_SSO_Cookie>.
 
-This module provides the ability to authenticate RT users
-against one or more external data sources at once. It will
-also allow information about that user to be loaded from
-the same, or any other available, source as well as allowing
-multple redundant servers for each method.
+    Set($ExternalAuthPriority,  [ 'My_LDAP',
+                                  'My_MySQL',
+                                  'My_SSO_Cookie'
+                                ]
+    );
 
-The extension currently supports authentication and 
-information from LDAP via the Net::LDAP module, and from
-any data source that an installed DBI driver is available
-for. 
+=item C<$ExternalInfoPriority>
 
-It is also possible to use cookies set by an alternate
-application for Single Sign-On (SSO) with that application.
-For example, you may integrate RT with your own website login
-system so that once users log in to your website, they will be
-automagically logged in to RT when they access it.
+When multiple auth services are available, this value defines the order
+in which the services defined in L</$ExternalSettings> should be used to
+get information about users. This includes C<RealName>, telephone
+numbers etc, but also whether or not the user should be considered
+disabled.
 
-It was originally designed and tested against: 
+Once a user record is found, no more services are checked.
 
-MySQL v4.1.21-standard
-MySQL v5.0.22
-Windows Active Directory v2003
+You CANNOT use a SSO cookie to retrieve information.
 
-But it has been designed so that it should work with ANY
-LDAP service and ANY DBI-drivable database, based upon the
-configuration given in your $RTHOME/etc/RT_SiteConfig.pm
+You should remove services you don't use, but you must define
+at least one service.
 
-As of v0.08 ExternalAuth also allows you to pull a browser
-cookie value and test it against a DBI data source allowing
-the use of cookies for Single Sign-On (SSO) authentication
-with another application or website login system. This is
-due to the merging of RT::Authen::ExternalAuth and
-RT::Authen::CookieAuth. For example, you may integrate RT
-with your own website login system so that once users log in
-to your website, they will be automagically logged in to RT 
-when they access it.
+    Set($ExternalInfoPriority,  [ 'My_LDAP',
+                                  'My_MySQL',
+                                ]
+    );
 
+=item C<$AutoCreateNonExternalUsers>
 
-=head1 INSTALLATION
+If this is set to 1, then users should be autocreated by RT
+as internal users if they fail to authenticate from an
+external service. This is useful if you have users outside
+your organization who might interface with RT, perhaps by sending
+email to a support email address.
 
-To install this module, run the following commands:
+=item C<$ExternalSettings>
 
-    perl Makefile.PL
-    make
-    make install
+These are the full settings for each external service as a hash of
+hashes.  Note that you may have as many external services as you wish.
+They will be checked in the order specified in L</$ExternalAuthPriority>
+and L</$ExternalInfoPriority> directives above.
 
-If you are using RT 3.8.x, you need to enable this
-module by adding RT::Authen::ExternalAuth to your
-@Plugins configuration:
+The outer structure is a key with the authentication option (name of
+external source). The value is a hash reference with configuration keys
+and values, for example:
 
-    Set( @Plugins, qw(RT::Authen::ExternalAuth) );
+    Set($ExternalSettings, {
+        My_LDAP => {
+            type => 'ldap',
+            ... other options ...
+        },
+        My_MySQL => {
+            type => 'db',
+            ... other options ...
+        },
+        ... other sources ...
+    } );
 
-If you already have a @Plugins line, add RT::Authen::ExternalAuth to the
-existing list.  Adding a second @Plugins line will cause interesting
-bugs.
+As shown above, each description should have 'type' defined.
+The following types are supported:
 
-Once installed, you should view the file:
-    
-3.4/3.6    $RTHOME/local/etc/ExternalAuth/RT_SiteConfig.pm
-3.8        $RTHOME/local/plugins/RT-Authen-ExternalAuth/etc/RT_SiteConfig.pm
+=over 4
 
-Then use the examples provided to prepare your own custom 
-configuration which should be added to your site configuration in
-$RTHOME/etc/RT_SiteConfig.pm
+=item ldap
 
-=head1 AUTHOR
-        Mike Peachey
-        Jennic Ltd.
-        zordrak@cpan.org
+Authenticate against and sync information with LDAP servers.  See
+L<RT::Authen::ExternalAuth::LDAP> for details.
 
-        Various Best Practical Developers
+=item db
 
-=head1 COPYRIGHT AND LICENCE
+Authenticate against and sync information with external RDBMS, supported
+by Perl's L<DBI> interface. See L<RT::Authen::ExternalAuth::DBI> for
+details.
 
-Copyright (C) 2008, Jennic Ltd.
+=item cookie
 
-This software is released under version 2 of the GNU 
-General Public License. The license is distributed with
-this package in the LICENSE file found in the directory 
-root.
+Authenticate by cookie. See L<RT::Authen::ExternalAuth::DBI::Cookie> for
+details.
+
+=back
+
+See the modules noted above for configuration options specific to each
+type.  The following apply to all types.
+
+=over 4
+
+=item attr_match_list
+
+The list of RT attributes that uniquely identify a user. These values
+are used, in order, to find users in the selected authentication
+source. Each value specified here must have a mapping in the
+L</attr_map> section below. You can remove values you don't expect to
+match, but we recommend using C<Name> and C<EmailAddress> at a
+minimum. For example:
+
+    'attr_match_list' => [
+        'Name',
+        'EmailAddress',
+    ],
+
+You should not use items that can map to multiple users (such as a
+C<RealName> or building name).
+
+=item attr_map
+
+Mapping of RT attributes on to attributes in the external source.
+Valid keys are attributes of an
+L<RT::User|http://bestpractical.com/rt/docs/latest/RT/User.html>.
+The values are attributes from your authentication source.
+For example, an LDAP mapping might look like:
+
+    'attr_map' => {
+        'Name'         => 'sAMAccountName',
+        'EmailAddress' => 'mail',
+        'Organization' => 'physicalDeliveryOfficeName',
+        'RealName'     => 'cn',
+        ...
+    },
+
+=back
+
+=back
+
+=head2 Example
+
+    # Use the below LDAP source for both authentication, as well as user
+    # information
+    Set( $ExternalAuthPriority, ["My_LDAP"] );
+    Set( $ExternalAuthInfo,     ["My_LDAP"] );
+
+    # Users created from LDAP should be Privileged; this is a core RT
+    # option.  Additionally, this is the 4.2 name for the option; for RT
+    # 4.0, is it named $AutoCreate   See the core RT documentation at
+    # http://docs.bestpractical.com/RT_Config#UserAutocreateDefaultsOnLogin
+    # for for further details.
+    Set( $UserAutocreateDefaultsOnLogin, { Privileged => 1 } );
+
+    # Users should still be autocreated by RT as internal users if they
+    # fail to exist in an external service; this is so requestors (who
+    # are not in LDAP) can still be created when they email in.
+    Set($AutoCreateNonExternalUsers, 1);
+
+    # Minimal LDAP configuration; see RT::Authen::ExternalAuth::LDAP for
+    # further details and examples
+    Set($ExternalSettings, {
+        'My_LDAP'       =>  {
+            'type'             =>  'ldap',
+            'server'           =>  'ldap.example.com',
+            # By not passing 'user' and 'pass' we are using an anonymous
+            # bind, which some servers to not allow
+            'base'             =>  'ou=Staff,dc=example,dc=com',
+            'filter'           =>  '(objectClass=inetOrgPerson)',
+            # Users are allowed to log in via email address or account
+            # name
+            'attr_match_list'  => [
+                'Name',
+                'EmailAddress',
+            ],
+            # Import the following properties of the user from LDAP upon
+            # login
+            'attr_map' => {
+                'Name'         => 'sAMAccountName',
+                'EmailAddress' => 'mail',
+                'RealName'     => 'cn',
+                'WorkPhone'    => 'telephoneNumber',
+                'Address1'     => 'streetAddress',
+                'City'         => 'l',
+                'State'        => 'st',
+                'Zip'          => 'postalCode',
+                'Country'      => 'co',
+            },
+        },
+    } );
+
+=head1 AUTHORS
+
+Best Practical Solutions, LLC E<lt>modules@bestpractical.comE<gt>
+
+Originally by Mike Peachey (Jennic Ltd.) <zordrak@cpan.org>
+
+=head1 BUGS
+
+All bugs should be reported via email to
+
+    L<bug-RT-Authen-ExternalAuth@rt.cpan.org|mailto:bug-RT-Authen-ExternalAuth@rt.cpan.org>
+
+or via the web at
+
+    L<rt.cpan.org|http://rt.cpan.org/Public/Dist/Display.html?Name=RT-Authen-ExternalAuth>.
+
+=head1 LICENSE AND COPYRIGHT
+
+Copyright (c) 2008-2014 by Best Practical Solutions, LLC
+Copyright (c) 2008 by Jennic Ltd.
+
+This is free software, licensed under:
+
+  The GNU General Public License, Version 2, June 1991
 
 =cut
 
